#include <proto.h>

#include "gtdialog.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtDialog class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init dialog.
*
\************************************************************************/
GtDialog *GtMkDialog(char *n,GtManager *p,GtTag *tgs)
{
	GtDialog *obj;
	
	if(!(obj=new GtDialog()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtDialog object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init dialog.
*
\************************************************************************/
GtDialog *GtMkDialog(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtDialog *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkDialog(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}
	
/************************************************************************\
*
* Setup empty dialog.
*
\************************************************************************/
GtDialog::GtDialog(void)
{
	block=FALSE;
	flags&=~GTWF_MAPPED;
}

/************************************************************************\
*
* Setup dialog.
*
\************************************************************************/
sbool GtDialog::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	
	if(!GtAppShell::Init(n,p,tgs))
		return(FALSE);
	if(!p || !p->IsSubClassOf("GtWidget"))
	{
		GtError(this,GTEC_MISC,"No parent for dialog!\n");
		return(FALSE);
	}
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Block:
				if(t->d.val)
					block=TRUE;
				else
					block=FALSE;
				break;
		}
	}
	return(TRUE);
}

/************************************************************************\
*
* Set dialog attributes.
*
\************************************************************************/
sbool GtDialog::Set(GtTag *tgs)
{
	GtTag *t;
	
	if(!GtAppShell::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Block:
				if(t->d.val)
					block=TRUE;
				else
					block=FALSE;
				break;
		}
	}
	return(TRUE);
}

/************************************************************************\
*
* Get dialog attributes.
*
\************************************************************************/
void GtDialog::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtAppShell::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Block:
				if(block)
					*ptr=TRUE;
				else
					*ptr=FALSE;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtDialog::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtDialog")==0)
		return(TRUE);
	else
		return(GtAppShell::IsSubClassOf(cn));
}

/************************************************************************\
*
* Map or unmap window and grab or ungrab events.
*
\************************************************************************/
void GtDialog::Map(sbool map)
{
	GtAppShell::Map(map);
	if(flags & GTWF_MAPPED)
	{
		if(block)
			dsp->SetGrab(this);
	}
	else
		dsp->ClrGrab(this);
}
		
/************************************************************************\
*
* Build window and set properties.
*
\************************************************************************/
sbool GtDialog::Build(void)
{
	if(flags & GTWF_UNBUILT)
	{
		grp=parent->GetWin()->XWin();
		if(grp)
			XSetTransientForHint(dsp->XDsp(),gtwin->XWin(),grp);
	}
	return(GtAppShell::Build());
}

/************************************************************************\
*
* Handle events and block other widgets if block set.
*
\************************************************************************/
int GtDialog::Handle(XEvent *xe)
{
	int ret;
	
	ret=GtAppShell::Handle(xe);
	if(block)
	{
		if(xe->type!=Expose)
			return(GTHR_USED);
	}
	return(ret);
}

/************************************************************************\
*
* Show dialog centered on parent.
*
\************************************************************************/
void GtDialog::Show(void)
{
	int xp=0,yp=0,xs=0,ys=0,nxp,nyp;
	Window dmy;
	
	XTranslateCoordinates(dsp->XDsp(),parent->GetWin()->XWin(),
												dsp->XDefaultRootWindow(),0,0,&xp,&yp,&dmy);
	parent->GetV(GT_XSize,&xs,GT_YSize,&ys,GT_DONE);
	nxp=xp+(xs-xsize)/2;
	nyp=yp+(ys-ysize)/2;
	this->Move(nxp,nyp);
	this->Map(TRUE);
}

/************************************************************************\
*
* Show dialog at given position.
*
\************************************************************************/
void GtDialog::Show(int xp,int yp)
{
	this->Move(xp,yp);
	this->Map(TRUE);
}

/************************************************************************\
*
* Show dialog centered on given widget.
*
\************************************************************************/
void GtDialog::Show(GtWidget *on)
{
	int xp=0,yp=0,xs=0,ys=0,nxp,nyp;
	Window dmy;
	
	XTranslateCoordinates(dsp->XDsp(),on->GetWin()->XWin(),
												dsp->XDefaultRootWindow(),0,0,&xp,&yp,&dmy);
	on->GetV(GT_XSize,&xs,GT_YSize,&ys,GT_DONE);
	nxp=xp+(xs-xsize)/2;
	nyp=yp+(ys-ysize)/2;
	this->Move(nxp,nyp);
	this->Map(TRUE);
}

/************************************************************************\
*
* Hide dialog.
*
\************************************************************************/
void GtDialog::Hide(void)
{
	this->Map(FALSE);
}
